# Import necessary libraries
import sys
import matplotlib.pyplot as plt

# Import modules from different packages
from .mce_2D_Operations.data_2d_operation import mce
from .mce_3D_Distribution.data_3d_plotting import mce_3d
from .MH_Interpolation.interpol import interpol

# Suppress warnings if not already suppressed
if not sys.warnoptions:
    import warnings
    warnings.simplefilter("ignore")

# Customize font size for matplotlib
plt.rcParams.update({'font.size': 7})





import itertools

def color_marker():
    """
    Generate lists of colors and markers for plotting purposes.

    Returns:
        colour (list): List of color strings representing different colors for plots.
        marker (itertools.cycle): An iterator that cycles through different marker symbols for plots.
    """
    colour = itertools.cycle(['b', 'g', 'r', 'c', 'm', 'y', 'k', 'tab:orange', 'tab:gray', 'tab:brown', 'tab:blue'])
    marker = itertools.cycle(['^', 'o', 'v', '*', '<', 'p', '>', 'h', 'P', 'H', 'X'])

    print ("</> colors and markers are generated  -----> using itertools.cycle() for [colour] and [marker]")
    return colour, marker


import numpy as np

def delSm_Pivoting(n, three_entropy_change_con, Label_one, temperatures):
    """
    Calculate the entropy changes based on the provided data.

    Parameters:
        n (int): Number of temperature values.
        three_entropy_change_con (list): List of lists containing entropy change data.
        Label_one (list): List of magnetic field values in Tesla.
        temperatures (list): List of temperature values.

    Returns:
        five_entropy_change_con (list): List of lists containing entropy change data in (10^-4 J/mol K) units.
        six_entropy_change_con (list): List of lists containing entropy change data along with temperature values.
    """
    six_entropy_change_con = []
    six_entropy_change_con.append(temperatures)
    five_entropy_change_con = []

    for j in range(0, len(Label_one), 1):
        four_entropy_change_con = []

        for i in range(0, n - 1, 1):
            four_entropy_change_con.append((three_entropy_change_con[i])[j])

        five_entropy_change_con.append(four_entropy_change_con)
        six_entropy_change_con.append(four_entropy_change_con)

    print ("</> data -----> pivoted")

    return five_entropy_change_con, six_entropy_change_con



import numpy as np
def delSm_T(n, one_n, two_n, H, M, T):
    """
    Calculate entropy changes for different temperatures and magnetic fields.

    Parameters:
        n (int): Number of temperature values.
        one_n (int): Total number of data samples per magnetic field.
        two_n (int): Total number of data samples for all magnetic fields.
        H (list): List of external magnetic field values.
        M (list): List of magnetization values.
        T (list): List of temperature values.

    Returns:
        three_entropy_change_con (list): A list of lists containing the calculated entropy changes.
        temperatures (list): A list of temperature values used for calculating the entropy changes.
        Label_one (list): A list of magnetic field values (in Tesla * 10^-4) for which entropy changes were calculated.
    """
    # Initialize variables to store entropy changes and temperatures
    three_entropy_change_con = []
    temperatures = []

    # Calculate the multiplier for magnetic field values
    one_n_max = one_n - 2
    #Multiplier = (H[one_n_max] - H[0]) / 10
    
    # Loop through temperature values to calculate entropy changes
    for q in range(0, n - 1, 1):
        one_entropy_change_con = 0
        two_entropy_change_con = []
        Label_one = []

        # Calculate entropy change for each data point in the one_n-1 range
        for j, i in zip(range(0, (one_n - 1), 1), range(q, two_n, n)):
            entropy_change = ((float(M[i + 1]) - float(M[i])) / (float(T[i + 1]) - float(T[i]))) * abs(float(H[j + 1]) - float(H[j]))

            one_entropy_change_con += entropy_change
            one_entropy_change_con = float(one_entropy_change_con)

            #j_th_field = H[j]
            #remainder = j_th_field % Multiplier

            let = one_n - 1
            div = 15
            if one_n < 16 :
                print ("</> there is not enough Magnetic Field data available")
            else:
                pass
            let_by = int(let/div)
            if (j % let_by == 0) or (j == one_n - 2):
                two_entropy_change_con.append(one_entropy_change_con)
                Label_one.append(round((H[j]), 2))

        # Store calculated entropy changes and temperature values for each iteration
        temperatures.append(float(T[q]))
        three_entropy_change_con.append(two_entropy_change_con)

    # Return the final calculated entropy changes, temperature values, and magnetic field labels
    return three_entropy_change_con, temperatures, Label_one


import xlrd
import math
import xlsxwriter
import numpy as np
import matplotlib.pyplot as plt


def interpol(path, sheet_index, T_row, H_col, int_val, final_val, interval, interpol_type, interpol_mode, deg):

    def column_letter_to_number(column_letter):
        column_number = 0
        for char in column_letter:
            # Convert the character to its ASCII value and subtract 64
            # to get the 1-based index (A=1, B=2, ..., Z=26)
            column_number = column_number * 26 + (ord(char.upper()) - 64)
        return column_number


    # Open the Excel workbook
    workbook = xlrd.open_workbook(path)
    sheet = workbook.sheet_by_index(sheet_index-1)

    n_rows = sheet.nrows
    n_cols = sheet.ncols

    prov_data = []

    prov_T = []
    # Extract temperature values from the first row
    for i in range(n_cols):
        prov_T.append(sheet.cell_value((int(T_row)-1), i))

    # Extract data from the Excel sheet
    for i in range(n_cols):
        col_data = []
        for j in range(1, n_rows):
            col_data.append(sheet.cell_value(j, i))

        prov_data.append(col_data)

    H_col_num = column_letter_to_number(H_col)


    prov_H = prov_data.pop(int(H_col_num) -1)


    # Define a function to check if the last element of a list is a string
    def check_last_element(prov_data):
        last_element = prov_data[0][-1]
        if isinstance(last_element, str):
            return True
        else:
            return False

    # If the last element of prov_data is a string, remove it from each list
    if check_last_element(prov_data):
        for i in range(0, len(prov_data)):
            prov_data[i].pop(len(prov_data[i]) - 1)
        prov_H.pop(len(prov_H) - 1)
    else:
        pass

    H_num = int(((final_val - int_val)/interval) + 1)	
    H_np = np.linspace(int(int_val), int(final_val), H_num) 
    H = H_np.tolist() 

    if (interpol_type == 'poly'):
        best_degree_con = []
        # Define a range of polynomial degrees to test
        if (interpol_mode == 'auto'):
            degrees_to_test = range(1, 50)  # Test degrees from 1 to 49   
            se_con_final = [] 
            for i in range(0, len(prov_data)):
                best_degree = 1
                best_mse = np.inf
                for degree in degrees_to_test:
                    
                    coef = np.polyfit(prov_H, prov_data[i], degree)
                    poly1d_fn = np.poly1d(coef)

                    # Calculate the predicted values
                    prov_data_pred = poly1d_fn(H)
                    prov_data_intt = np.interp(H, prov_H, prov_data[i])
                    # Calculate the Mean Squared Error (MSE)
                    mse = np.mean((prov_data_intt - prov_data_pred) ** 2)
                    se_con = ((prov_data_intt - prov_data_pred) ** 2)


                    # Check if this degree has a lower MSE
                    if mse < best_mse:
                        best_mse = mse
                        best_degree = degree
                #se_con_final.append(se_con)        
                best_degree_con.append(best_degree)
        else:
            for i in range(0, len(prov_data)):
                best_degree = deg
                best_degree_con.append(best_degree)
        # Fit the best degree polynomial to the data
        M_fitted_con = []
        se_con_final = [] 

        for i in range(0, len(prov_data)):

            best_coef = np.polyfit(prov_H, prov_data[i], best_degree_con[i])
            prov_data_fn = np.poly1d(best_coef)
            M_fitted = prov_data_fn(H)
            prov_data_intt = np.interp(H, prov_H, prov_data[i])
            # Calculate the Mean Squared Error (MSE)
            mse = np.mean((prov_data_intt - M_fitted) ** 2)
            se_con = ((prov_data_intt - M_fitted) ** 2)
            se_con_final.append(se_con)
            M_fitted = M_fitted.tolist()
            M_fitted_con.append(M_fitted)
        total_interp_M_con = M_fitted_con

        plt.pcolor(se_con_final, cmap='terrain')

        # Add colorbar with a custom label
        cbar = plt.colorbar(orientation='horizontal')
        cbar.set_label('Square Error', fontname='monospace')

        # Set labels for the X and Y axes with monospace font
        plt.xlabel('x-axis (sheet_col)', fontname='monospace')
        plt.ylabel('y-axis (sheet_row)', fontname='monospace')

        # Show the plot
        plt.show()


    elif (interpol_type == 'lin'):    
        total_interp_M_con = []
        se_con_final = [] 
        for m in range(0, len(prov_data)):
            interp_M_con = np.interp(H, prov_H, prov_data[m])
            prov_data_intt = np.interp(H, prov_H, prov_data[m])
            # Calculate the Mean Squared Error (MSE)
            mse = np.mean((prov_data_intt - interp_M_con ) ** 2)
            se_con = ((prov_data_intt - interp_M_con) ** 2)
            se_con_final.append(se_con)
            interp_M_con_list = interp_M_con.tolist()
            total_interp_M_con.append(interp_M_con_list)

        plt.pcolor(se_con_final, cmap='terrain')

        # Add colorbar with a custom label
        cbar = plt.colorbar(orientation='horizontal')
        cbar.set_label('Square Error', fontname='monospace')

        # Set labels for the X and Y axes with monospace font
        plt.xlabel('x-axis (sheet_col)', fontname='monospace')
        plt.ylabel('y-axis (sheet_row)', fontname='monospace')

        # Show the plot
        plt.show()

    else :
        print ("</> invalid 'interpol_type', choose either 'lin' or 'poly' ")


	
    total_interp_M_con.insert(0, H)

    for i in range(0, len(prov_T)):
        total_interp_M_con[i].insert(0, prov_T[i])

    final_data = total_interp_M_con
    # Write the interpolated data to a new Excel file
    workbook = xlsxwriter.Workbook(path)
    worksheet = workbook.add_worksheet()

    for col, col_data in enumerate(final_data):
        for row, value in enumerate(col_data):
            worksheet.write(row, col, value)
    workbook.close()

    print ("</> data -----> interpolated based on 'col' data")

    return ("")


def MH_Pivoting(one_n, n, M, H):
    """
    Reshape the magnetization and external field data for plotting.

    Args:
        one_n (int): Number of data points along one direction.
        n (int): Total number of data points.
        M (list): List of magnetization values.
        H (list): List of external field values.

    Returns:
        tuple: A tuple containing two lists:
               - one_M_plot_final (list): A list of lists containing magnetization data,
                 grouped by one direction.
               - two_M_plot_final (list): A list of lists containing magnetization data,
                 grouped by the other direction.
    """
    # Reshape the data for one direction (H/M data grouped by the other direction).
    one_M_plot_final = []
    for k in range(0, n, 1):
        one_M_plot = []
        for l in range(0, (one_n - 1), 1):
            index = (((l + 1) * (n - k)) + l * k) - 1
            one_M_plot.append(M[index])
        one_M_plot_final.append(one_M_plot)

    # Reshape the data for the other direction (H/M data grouped by one direction).
    two_M_plot_final = []
    for k in range(0, (one_n - 1), 1):
        two_M_plot = []
        for l in range(k * n, ((k + 1) * n) - 1, 1):
            two_M_plot.append(M[l])
        two_M_plot_final.append(two_M_plot)

    return one_M_plot_final, two_M_plot_final


import numpy as np

def n_expo(n, one_n, two_n, H, M, T):

    '''
        Parameters:
            n (int): Number of temperature values.
            one_n (int): Total number of data samples per magnetic field.
            two_n (int): Total number of data samples for all magnetic fields.
            H (list): List of external magnetic field values.
            M (list): List of magnetization values.
            T (list): List of temperature values.
    '''

    H_arr = H
    T_arr = [] 
    M_arr = []
    er_init = 0
    er_final = n
    for i in range(0, one_n):
            M_arr_prov = []
            for j in range (er_init, er_final):
                    M_arr_prov.append(M[j])
            M_arr.append(M_arr_prov)
            er_init += n
            er_final += n
    for i in range (0, n):
            T_arr.append(T[i])


    entropy_change_con = []
    sum_array = np.zeros(n-1)
    for i in range (0, one_n-1):
            entropy_change_con_prov = []
            for j in range (0, n-1):
                    entropy_change = ((float(M_arr[i][j + 1]) - float(M_arr[i][j])) / (float(T_arr[j + 1]) - float(T_arr[j]))) * abs(float(H_arr[i+1]) - float(H_arr[i])) 

                    entropy_change_con_prov.append(entropy_change)
            sum_array += np.asarray(entropy_change_con_prov)
            sum_list = sum_array.tolist()
            entropy_change_con.append(sum_list)
                    

    N_expo_con = []
    count = 0
    N_H_label = []
    for i in range (0, one_n-2):
            N_expo_con_prov = []
            for j in range (0, n-1):
                    N_expo =  (np.log(abs(float(entropy_change_con[i+1][j]))) - np.log(abs(float(entropy_change_con[i][j]))))/ (np.log(abs(float(H_arr[i+1]))) - np.log(abs(float(H_arr[i]))))
                    N_expo_con_prov.append(N_expo)


            let = one_n - 1
            div = 15
            if one_n < 16 :
                print ("</> there is not enough Magnetic Field data available")
            else:
                pass
            let_by = int(let/div)

            if (i % let_by == 0) or (i == one_n - 3):
                    N_expo_con.append(N_expo_con_prov)
                    count +=1
                    N_H_label.append(round(H_arr[i], 2))

    T_arr.pop()

    return N_expo_con, T_arr, H_arr, N_H_label
 







                





import numpy as np

def Msqr_HbyM(one_n, n, M, H, T, one_M_plot_final):
    """
    Calculate data for generating the Arrott plot.

    Args:
        one_n (int): Number of data points along one direction.
        n (int): Total number of data points.
        M (list): List of magnetization values.
        H (list): List of external field values.
        T (list): List of temperature values.
        one_M_plot_final (list): List of lists containing magnetization data
                                 grouped by one direction.

    Returns:
        tuple: A tuple containing three lists:
               - H_plot_final (list): List of external field values without the last one.
               - M_sqr (list): List of squared magnetization values (M^2).
               - one_H_by_M_con (list): List of lists containing H/M (applied field / magnetization)
                                        values grouped by one direction.
    """
    one_n_pop = one_n - 1
    H_plot_final = H.copy()  # Create a copy to avoid modifying the original list.
    H_plot_final.pop(one_n_pop)

    M_sqr = np.square(one_M_plot_final)
    one_H_by_M_con = []

    for j in range(0, n, 1):
        two_H_by_M_con = []
        for i in range(0, one_n - 1, 1):
            H_by_M_val = H_plot_final[i] / one_M_plot_final[j][i]
            two_H_by_M_con.append(H_by_M_val)
        one_H_by_M_con.append(two_H_by_M_con)

    return H_plot_final, M_sqr, one_H_by_M_con


import matplotlib.pyplot as plt
import sys
from .Msqr_HbyM import Msqr_HbyM
from .data_reading import data_reading
from .color_marker import color_marker
from .data_plotting import data_plotting
from .delSm_T import delSm_T
from .delSm_Pivoting import delSm_Pivoting
from .MH_Pivoting import MH_Pivoting
from .arrott_plotting import arrott_plotting
from .data_writing import data_writing
from .T_FWHM_RCP import T_FWHM_RCP
from .mag_sus import mag_sus
from .N_exponent import n_expo

if not sys.warnoptions:
    import warnings
    warnings.simplefilter("ignore")

plt.rcParams.update({'font.size': 7})


def mce(samp_name, file_dir, sheet_index, T_row, H_col, g_name, M_unit, H_unit, T_unit, plot_legend, loc, field, linear_threshold, save_data):
    """
    Perform Magnetocaloric Effect (MCE) analysis.
    Args:
        n (int): Number of temperature values.
        one_n (int): Number of data points at each temperature.
    Returns:
        None: The function performs the MCE analysis and generates plots, without returning any value.
    """
    # Data Collection
    H, M, T, n, one_n, two_n = data_reading(file_dir, sheet_index, T_row, H_col)    
    # Entropy Change Calculation
    three_entropy_change_con, temperatures, Label_one = delSm_T(n, one_n, two_n, H, M, T)
    five_entropy_change_con, six_entropy_change_con = delSm_Pivoting(n, three_entropy_change_con, Label_one, temperatures)
    # Local Exponent Calculation
    N_expo_con, T_arr, H_arr, N_H_label = n_expo(n, one_n, two_n, H, M, T)
    # Color and Marker Definitions
    colour, marker = color_marker()
    # Magnetization and Field Reshaping
    one_M_plot_final, two_M_plot_final = MH_Pivoting(one_n, n, M, H)
    # Arrott Plot
    H_plot_final, M_sqr, one_H_by_M_con = Msqr_HbyM(one_n, n, M, H, T, one_M_plot_final)
    # Susceptibility
    susceptibility_final = []
    T_sus = []
    sus_inv = [] 
    sus = [] 
    np_T_sus = [] 
    regression_line = [] 
    np_T_sus_bu = [] 
    regression_bu = []
    x = 0
    if (g_name== 'sus_plot' or g_name== 'all_plots'):
        susceptibility_final, T_sus, sus_inv, sus, np_T_sus, regression_line, np_T_sus_bu, regression_bu, x = mag_sus(g_name, n, one_n, T, H, M, field, linear_threshold)
    else:
        pass
    T_FWHM_con = []
    T_FWHM_con_final = []
    RCP_con = []
    RCP_final = []
    H_for_RCP = []
    if (g_name== 'RCP_plot' or g_name== 'all_plots'):       
        # Calculate T_FWHM and RCP
        T_FWHM_con, T_FWHM_con_final, RCP_con, RCP_final, H_for_RCP = T_FWHM_RCP(n, Label_one, six_entropy_change_con)
    else:
        pass
    # Data Visualization
    data_plotting(g_name, one_n, n, T, H, M_unit, H_unit, T_unit, colour, marker, Label_one, plot_legend, loc, one_M_plot_final, two_M_plot_final, H_plot_final, temperatures, five_entropy_change_con, M_sqr, one_H_by_M_con, N_expo_con, T_arr, N_H_label,  T_sus, sus_inv, sus, np_T_sus, regression_line, np_T_sus_bu, regression_bu, x, linear_threshold, T_FWHM_con, T_FWHM_con_final, RCP_con, RCP_final, H_for_RCP, samp_name)
    # Modified Arrott Plot
    M_pow_MFT, H_by_M_pow_MFT, M_pow_TMFT, H_by_M_pow_TMFT, M_pow_3DH, H_by_M_pow_3DH, M_pow_3DI, H_by_M_pow_3DI = arrott_plotting(g_name, n, one_M_plot_final, one_H_by_M_con)
    # Store Data to Excel Files
    if (save_data == 'allow'):
        data_writing(g_name, file_dir, n, T, Label_one, six_entropy_change_con, M_sqr, one_H_by_M_con, M_pow_MFT, H_by_M_pow_MFT, M_pow_TMFT, H_by_M_pow_TMFT, M_pow_3DH, H_by_M_pow_3DH, M_pow_3DI, H_by_M_pow_3DI, susceptibility_final, T_FWHM_con_final, RCP_final, H_for_RCP, N_expo_con, T_arr, N_H_label)
    else:
        pass
    return ("")

       




import matplotlib.pyplot as plt
import numpy as np

def arrott_plotting(g_name, n, one_M_plot_final, one_H_by_M_con):
    """
    Generate modified Arrott plots using Matplotlib.

    Args:
        n (int): Number of data points.
        one_M_plot_final (list): List of lists containing magnetization data.
        one_H_by_M_con (list): List of lists containing H/M (applied field / magnetization) data.

    Returns:
        None: The function displays four subplots of modified Arrott plots.
    """
    # Calculate powers of magnetization and H/M for four different β and γ values.
    M_pow_MFT = np.power(one_M_plot_final, 2)
    H_by_M_pow_MFT = np.power(one_H_by_M_con, 1)

    M_pow_TMFT = np.power(one_M_plot_final, 4)
    H_by_M_pow_TMFT = np.power(one_H_by_M_con, 1)

    M_pow_3DH = np.power(one_M_plot_final, (1 / 0.365))
    H_by_M_pow_3DH = np.power(one_H_by_M_con, (1 / 1.336))

    M_pow_3DI = np.power(one_M_plot_final, (1 / 0.325))
    H_by_M_pow_3DI = np.power(one_H_by_M_con, (1 / 1.24))

    if (g_name== 'arrott_plots' or g_name== 'all_plots'):
        # Create four subplots to display the modified Arrott plots.
        plt.subplot(2, 2, 1)
        for i in range(n):
            plt.plot(H_by_M_pow_MFT[i], M_pow_MFT[i], linestyle='solid', marker='.', markersize=2, linewidth= 0.5)
        plt.xlabel("(H/M)^(1/γ)")
        plt.ylabel("M^(1/β)")
        plt.title("Arrott plot 01 (β:0.5; γ:1)")

        plt.subplot(2, 2, 2)
        for i in range(n):
            plt.plot(H_by_M_pow_TMFT[i], M_pow_TMFT[i], linestyle='solid', marker='.', markersize=2, linewidth= 0.5)
        plt.xlabel("(H/M)^(1/γ)")
        plt.ylabel("M^(1/β)")
        plt.title("Arrott plot 02 (β:0.25; γ:1)")

        plt.subplot(2, 2, 3)
        for i in range(n):
            plt.plot(H_by_M_pow_3DH[i], M_pow_3DH[i], linestyle='solid', marker='.', markersize=2, linewidth= 0.5)
        plt.xlabel("(H/M)^(1/γ)")
        plt.ylabel("M^(1/β)")
        plt.title("Arrott plot 03 (β:0.365; γ:1.336)")

        plt.subplot(2, 2, 4)
        for i in range(n):
            plt.plot(H_by_M_pow_3DI[i], M_pow_3DI[i], linestyle='solid', marker='.', markersize=2, linewidth= 0.5)
        plt.xlabel("(H/M)^(1/γ)")
        plt.ylabel("M^(1/β)")
        plt.title("Arrott plot 04 (β:0.325; γ:1.24)")

        # Adjust the layout and display the subplots.
        plt.tight_layout()
        plt.show()

        print ("</> modified arrott plots successfully generated using [one_M_plot_final] and [one_H_by_M_con] with different power impositions")

    return M_pow_MFT, H_by_M_pow_MFT, M_pow_TMFT, H_by_M_pow_TMFT, M_pow_3DH, H_by_M_pow_3DH, M_pow_3DI, H_by_M_pow_3DI



import numpy as np
from scipy import stats
import matplotlib.pyplot as plt

def mag_sus(g_name, n, one_n, T, H, M, field, linear_threshold):
	T_sus = [T[i] for i in range(0, n)]
	M_sus_con = []
	mm = 0
	nn = n
	for j in range(0, one_n):
		M_sus = []
		for i in range(mm, nn):
			M_sus.append(M[i])
		mm += n	
		nn += n	
		M_sus_con.append(M_sus)


	sus_head = []
	arrow = []	
	sus_head_int = []
	for i in range(0, n):
		sus_head_int.append(str(T_sus[i]))
		arrow.append('↓')
	sus_head.append(sus_head_int)
	sus_head[0].insert(0, 'Temperature')
	sus_head[0].insert(1, '→')
	arrow.insert(0, 'Field(H)')
	arrow.insert(1, '')
	sus_head.append(arrow)

	last_array = M_sus_con[-1]
	contains_string = any(isinstance(item, str) for item in last_array)

	# If there's at least one string element, remove the last array
	if contains_string:
	    M_sus_con.pop()
	    sus_sus_inv_con = []

	    for i in range(0, one_n-1):          

	        sus_inv_store = []
	        sus_store = []
	        for j in range(0, n):
	            sus_inv_store.append(str(H[i]/(M_sus_con[i])[j]))
	            sus_store.append(str(M_sus_con[i][j]/H[i]))   
	        sus_sus_inv_con.append(sus_inv_store)
	        sus_sus_inv_con.append(sus_store)  
	    for i, j, k in zip(range(0, 2*(one_n-1), 2), range(1, 2*(one_n-1), 2), range(0, (one_n-1))):
	        sus_sus_inv_con[i].insert(0, str(H[k]))
	        sus_sus_inv_con[i].insert(1, f"1/Chi →")
	        sus_sus_inv_con[j].insert(0, ' ')
	        sus_sus_inv_con[j].insert(1, 'Chi →')

	    for i in range(0, 2*(one_n-1)):
	        sus_head.append(sus_sus_inv_con[i])


	else:
	    sus_sus_inv_con =[]
	    for i in range(0, one_n):
	        sus_inv_store = []
	        sus_store = []
	        for j in range(0, n):
	            sus_inv_store.append(str(H[i]/(M_sus_con[i])[j]))
	            sus_store.append(str(M_sus_con[i][j]/H[i]))	
	        sus_sus_inv_con.append(sus_inv_store)
	        sus_sus_inv_con.append(sus_store)
	    for i,j,k in zip(range(0, 2*one_n, 2), range(1, 2*one_n, 2), range(0, one_n)):
	        sus_sus_inv_con[i].insert(0 , str(H[k]))
	        sus_sus_inv_con[i].insert(1 , f"1/Chi →")
	        sus_sus_inv_con[j].insert(0 , ' ')
	        sus_sus_inv_con[j].insert(1 , 'Chi →')		

	    for i in range(0, 2*one_n):
	            sus_head.append(sus_sus_inv_con[i])


	susceptibility_final = sus_head

	H_ind = None
	for i in range(0, one_n):
		if field == str(H[i]):
			H_ind = i
			break
			
	sus_inv = [H[int(H_ind)]/M_sus_con[int(H_ind)][i] for i in range(0, n)]
	sus = [M_sus_con[int(H_ind)][i]/H[int(H_ind)] for i in range(0, n)]


	T_sus_inp = np.linspace(min(T_sus), max(T_sus), 100)
	sus_inv_interpol = np.interp(T_sus_inp, T_sus, sus_inv)
	sus_interpol = np.interp(T_sus_inp, T_sus, sus)

	ele = 0
	for l in range (2, n):
		sus_inv_last = sus_inv[-l:]
		T_sus_last = T_sus[-l:]
		slope, intercept, r_value, p_value, std_err = stats.linregress(T_sus_last, sus_inv_last)
		if (linear_threshold >= r_value):
			ele = (l)
			break

	np_T_sus = np.asarray(T_sus[-ele:])		
	regression_line = float(slope) * np_T_sus + float(intercept)

	np_T_sus_bu = np_T_sus
	regression_bu = sus_inv[-ele:]
	x = float((round(min(sus), 2) - float(intercept))/float(slope))
	regression_line = np.insert(regression_line, 0, 0)
	np_T_sus = np.insert(np_T_sus, 0, x)



	return susceptibility_final, T_sus, sus_inv, sus, np_T_sus, regression_line, np_T_sus_bu, regression_bu, x






import xlrd

def data_reading(file_dir, sheet_index, T_row, H_col):
    """
    Collects magnetization (M) and external fields (H) data from an Excel sheet.

    Parameters:
        file_dir (str): The file path of the Excel sheet.
        n (int): The number of temperature values (columns) in the Excel sheet.
        one_n (int): The number of data rows for each field value (H) in the Excel sheet.
        T (list): A list containing the temperature values.

    Returns:
        H (list): A list containing the external fields data.
        M (list): A list containing the magnetization data.
    """

    # Initialize empty lists to store magnetization (M) and external fields (H) data
    Magnetization_val = []
    External_Fields = []
    M = Magnetization_val
    H = External_Fields

    # Open the Excel file and read the data
    book = xlrd.open_workbook(file_dir)
    sheet = book.sheet_by_index(sheet_index-1)
    data = [[sheet.cell_value(r, c) for c in range(sheet.ncols)] for r in range(sheet.nrows)]

    def column_letter_to_number(column_letter):
        column_number = 0
        for char in column_letter:
            # Convert the character to its ASCII value and subtract 64
            # to get the 1-based index (A=1, B=2, ..., Z=26)
            column_number = column_number * 26 + (ord(char.upper()) - 64)
        return column_number
    H_col_num = column_letter_to_number(H_col)
    T = data[T_row-1] 
    T.pop(0)
    # Loop to collect magnetization (M) and external fields (H) data from the Excel sheet
    for a in range(T_row, sheet.nrows, 1):
        H.append((data[a])[int(H_col_num)-1])
        for b in range(0, sheet.ncols-1):
            T.append(T[b])
        for b in range(H_col_num, sheet.ncols):
            M.append((data[a])[b])   
    two_n =  int((sheet.ncols-1)*(sheet.nrows-1))
    n = int(sheet.ncols-1)
    one_n = int(sheet.nrows-1)

    print ("</> source file accessed ---> cell_values successfuly stored in [data] \n ---> [M]<-->[H]<-->[T] seperated properly")
    # Return the collected external fields (H) and magnetization (M) data as lists
    return H, M, T, n, one_n, two_n


# Import necessary libraries
import xlrd
import math
import numpy as np
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.pylab import *
from mpl_toolkits.mplot3d import Axes3D
import xlsxwriter
import openpyxl
import os

def mce_3d(path, sheet_index, T_row, H_col, g_name, M_unit, H_unit, T_unit, dpi, save_data):


    folder_path = os.path.dirname(path)

    # Create a new Excel workbook


    # Write data to the worksheet
    #worksheet['A1'] = 'Hello'
    #worksheet['B1'] = 'World'

    new_folder_name = 'mce_output'
    new_folder_path = os.path.join(folder_path, new_folder_name)

    # Check if the folder already exists, and create it if not
    if not os.path.exists(new_folder_path):
        os.makedirs(new_folder_path)
        print(f"</> folder '{new_folder_name}' created at '{new_folder_path}'")
    else:
        print(f"</> folder '{new_folder_name}' already exists at '{new_folder_path}'")

    # Create the full file path including the folder and filename




    def column_letter_to_number(column_letter):
        column_number = 0
        for char in column_letter:
            # Convert the character to its ASCII value and subtract 64
            # to get the 1-based index (A=1, B=2, ..., Z=26)
            column_number = column_number * 26 + (ord(char.upper()) - 64)
        return column_number

    # Open the Excel workbook
    workbook = xlrd.open_workbook(path)
    sheet = workbook.sheet_by_index(sheet_index-1)

    # Get the number of rows and columns in the Excel sheet
    n_rows = sheet.nrows
    n_cols = sheet.ncols

    # Initialize data containers
    init_data = []

    # Initialize lists for temperature and applied field values
    prov_T = []
    for i in range(n_cols):
        prov_T.append(sheet.cell_value((int(T_row)-1), i))

    # Populate init_data with the Excel sheet data
    for i in range(n_cols):
        col_data = []
        for j in range(1, n_rows):
            col_data.append(sheet.cell_value(j, i))
        init_data.append(col_data)

    # Define a function to check if the last element of init_data is a string
    def check_last_element(init_data):
        last_element = init_data[0][-1]
        if isinstance(last_element, str):
            return True
        else:
            return False

    H_col_num = column_letter_to_number(H_col)
    # Remove the first element from prov_T, init_data to store field in H
    prov_T.pop(0)
    H = init_data.pop(int(H_col_num) -1)
    T = prov_T
    M = init_data

    # If the last element of init_data is a string, remove the last element from each list
    if check_last_element(init_data):
        for i in range(0, len(init_data)):
            init_data[i].pop(len(init_data[i]) - 1)
        H.pop(len(H) - 1)
    else:
        pass

    px = int((dpi)**(1/2))
    px_1 = int(px + 1) 
    # Create linearly spaced arrays for applied field (H_lin) and temperature (T_lin)
    H_lin = np.linspace(H[0], H[len(H) - 1], px_1)
    T_lin = np.linspace(T[0], T[len(T) - 1], px_1)

    # Initialize containers for interpolated data
    M_res_H = []
    M_res_HT = []

    # Interpolate data for applied field
    for i in range(0, len(M)):
        pri_M_res_H = np.interp(H_lin, H, M[i])
        nor_pri_M_res_H = pri_M_res_H.tolist()
        M_res_H.append(nor_pri_M_res_H)

    # Interpolate data for applied field and temperature
    for i in range(0, len(H_lin)):
        arrg_M_res_T = []
        for j in range(0, len(M)):
            arrg_M_res_T.append(M_res_H[j][i])
        pri_M_res_HT = np.interp(T_lin, T, arrg_M_res_T)
        nor_pri_M_res_HT = pri_M_res_HT.tolist()
        M_res_HT.append(nor_pri_M_res_HT)


    del_H = H_lin[1] - H_lin[0]
    entropy_change_con = []
    pri03_entropy_change_con = np.zeros(px)
    dMdT_con = []
    pri03_dMdT_con = np.zeros(px)

    # Calculate entropy change and derivative data
    for i in range(0, len(H_lin) - 1):
        pri02_entropy_change_con = []
        for j in range(0, len(T_lin) - 1):
            pri01_entropy_change = ((float(M_res_HT[i][j + 1]) - float(M_res_HT[i][j])) / (float(T_lin[j + 1]) - float(T_lin[j]))) * del_H 

            pri02_entropy_change_con.append(pri01_entropy_change)

        pri03_entropy_change_con = pri03_entropy_change_con + pri02_entropy_change_con
        nor_pri03_entropy_change_con = pri03_entropy_change_con.tolist()
        entropy_change_con.append(nor_pri03_entropy_change_con)

    for i in range(0, len(H_lin)):
        
        pri02_dMdT_con = []
        for j in range(0, len(T_lin) - 1):
            pri01_dMdT = ((float(M_res_HT[i][j + 1]) - float(M_res_HT[i][j])) / (float(T_lin[j + 1]) - float(T_lin[j])))

            pri02_dMdT_con.append(pri01_dMdT)

        pri03_dMdT_con = pri02_dMdT_con
        nor_pri03_dMdT_con = pri03_dMdT_con
        dMdT_con.append(nor_pri03_dMdT_con)


    dMdT_for_dMdTH = dMdT_con

    # Correct the order of entropy and derivative data
    entropy_change_con_correc = []
    dMdT_con_correc = []
    for i in range(0, len(M_res_HT)):
        corr_ind = len(entropy_change_con) - 1 - i
        entropy_change_con_correc.append(entropy_change_con[corr_ind])
        dMdT_con_correc.append(dMdT_con[corr_ind])

 

    dMdTH_con = []
    pri03_dMdTH = np.zeros(px)   

    for j in range(0, len(T_lin) - 1):
        pri02_dMdTH_con = []
        for i in range(0, len(H_lin) - 1):
            pri01_dMdTH = (((float(dMdT_for_dMdTH[i + 1][j])) - (float(dMdT_for_dMdTH[i][j])))/ (H_lin[i+1] - H_lin[i]))

            pri02_dMdTH_con.append(pri01_dMdTH)

        pri03_dMdTH_con = pri02_dMdTH_con
        nor_pri03_dMdTH_con = pri03_dMdTH_con
        dMdTH_con.append(nor_pri03_dMdTH_con)

    # Transpose and convert derivative data with respect to applied field
    dMdTH_con_correc_01 = np.transpose(dMdTH_con)
    dMdTH_con_correc_02 = dMdTH_con_correc_01.tolist()
    dMdTH_con_correc = []

    # Correct the order of derivative data with respect to applied field
    for i in range(0, len(dMdT_for_dMdTH)):
        corr_ind = len(entropy_change_con) - 1 - i
        dMdTH_con_correc.append(dMdTH_con_correc_02[corr_ind])




    # Initialize container for derivative data with respect to applied field
    dMdH_con = []
    pri03_dMdH_con = np.zeros(px)

    # Calculate derivative data with respect to applied field
    for j in range(0, len(T_lin)):
        pri02_dMdH_con = []
        for i in range(0, len(H_lin) - 1):
            pri01_dMdH = (((float(M_res_HT[i + 1][j])) - (float(M_res_HT[i][j])))/(H_lin[i+1] - H_lin[i]))

            pri02_dMdH_con.append(pri01_dMdH)

        pri03_dMdH_con = pri02_dMdH_con
        nor_pri03_dMdH_con = pri03_dMdH_con
        dMdH_con.append(nor_pri03_dMdH_con)

    # Transpose and convert derivative data with respect to applied field
    dMdH_con_correc_01 = np.transpose(dMdH_con)
    dMdH_con_correc_02 = dMdH_con_correc_01.tolist()
    dMdH_con_correc = []

    # Correct the order of derivative data with respect to applied field
    for i in range(0, len(M_res_HT)):
        corr_ind = len(entropy_change_con) - 1 - i
        dMdH_con_correc.append(dMdH_con_correc_02[corr_ind])


    if (g_name == 'MH_show'):        
        # Data for the first plot
        T_lin, H_lin = np.meshgrid(T_lin, H_lin)
        np_M_res_HT_correc = np.asarray(M_res_HT)

        # Create the first plot
        fig = plt.figure()
        ax1 = fig.add_subplot(111, projection='3d')

        # Plot the surface for the first dataset
        surface1 = ax1.plot_surface(T_lin, H_lin, np_M_res_HT_correc, cmap='plasma')
        ax1.set_xlabel(f'T {T_unit}')
        ax1.set_ylabel(f'H {H_unit}')
        ax1.set_zlabel(f'M {M_unit}')
        ax1.set_title('M distribution')
        cb = fig.colorbar(surface1, ax=ax1, label='M Value')
        ax = cb.ax
        text = ax.yaxis.label

        print ("</> request for MH_show ----> accepted & generated using the corrected form ([np_M_res_HT_correc]) with cmap = 'plasma'")
        # Show the first plot
        plt.show()

        if save_data == 'allow':

            workbook = openpyxl.Workbook()

            # Create a new worksheet
            worksheet = workbook.active

            file_path08 = os.path.join(folder_path, 'mce_output', 'obtained_MH_show.xlsx')
            workbook.save(file_path08)
            print ("</> workbook.save ----> [file_path08]")

            workbook.close()

            print ("</> workbook ----> closed")
            T_for_sheet = T_lin[0].tolist()
            H_for_sheet = []
            for i in range(0, len(H_lin)):
                H_for_sheet.append(H_lin[i][0])
            load_data = []
            T_for_sheet.insert(0, 'Temperature(T)')
            T_for_sheet.insert(1, '')
            ss_lst = ['↓'] * len(T_lin[0])
            ss_lst.insert(0, 'Field(H)')
            ss_lst.insert(1, '')

            load_data.append(T_for_sheet)
            load_data.append(ss_lst)

            
            for i in range(0, len(np_M_res_HT_correc)):
                load_data_prov = []
                load_data_prov = np_M_res_HT_correc[i].tolist()
                load_data_prov.insert(0, H_for_sheet[i])
                load_data_prov.insert(1, '→')  

                load_data.append(load_data_prov)              

            workbook = xlsxwriter.Workbook(file_path08)
            worksheet = workbook.add_worksheet()
            
            for row_num, row_data in enumerate(load_data):
                for col_num, cell_data in enumerate(row_data):
                    worksheet.write(row_num, col_num, cell_data)
            workbook.close()
            print("</> file accessed \n ---> data obtained ..... written correctly")
        else:
            pass


    elif(g_name == 'dSm_show'):    
        # Data for the second plot
        T_lin = np.delete(T_lin, -1)
        H_lin = np.delete(H_lin, -1)
        T_lin, H_lin = np.meshgrid(T_lin, H_lin)
        np_entropy_change_con_correc = np.asarray(entropy_change_con)

        # Create the second plot
        fig = plt.figure()
        ax2 = fig.add_subplot(111, projection='3d')

        # Plot the surface for the second dataset
        surface2 = ax2.plot_surface(T_lin, H_lin, np_entropy_change_con_correc, cmap='twilight')
        ax2.set_xlabel(f'T {T_unit}')
        ax2.set_ylabel(f'H {H_unit}')
        ax2.set_zlabel(f'∆Sm ({M_unit}).{H_unit}/{T_unit}')
        ax2.set_title('∆Sm distribution')
        cb = fig.colorbar(surface2, ax=ax2, label='∆Sm Value')
        ax = cb.ax
        text = ax.yaxis.label

        print ("</> request for dSm_show ----> accepted & generated using the corrected form ([np_entropy_change_con_correc]) with cmap = 'twilight'")
        # Show the second plot

        plt.show()

        if save_data == 'allow':

            workbook = openpyxl.Workbook()

            # Create a new worksheet
            worksheet = workbook.active

            file_path11= os.path.join(folder_path, 'mce_output', 'obtained_dSm_show.xlsx')
            workbook.save(file_path11)
            print ("</> workbook.save ----> [file_path11]")
            workbook.close()

            print ("</> workbook ----> closed")

            T_for_sheet = T_lin[0].tolist()
            H_for_sheet = []
            for i in range(0, len(H_lin)):
                H_for_sheet.append(H_lin[i][0])
            load_data = []
            T_for_sheet.insert(0, 'Temperature(T)')
            T_for_sheet.insert(1, '')
            ss_lst = ['↓'] * len(T_lin[0])
            ss_lst.insert(0, 'Field(H)')
            ss_lst.insert(1, '')

            load_data.append(T_for_sheet)
            load_data.append(ss_lst)

            
            for i in range(0, len(np_entropy_change_con_correc)):
                load_data_prov = []
                load_data_prov = np_entropy_change_con_correc[i].tolist()
                load_data_prov.insert(0, H_for_sheet[i])
                load_data_prov.insert(1, '→')  

                load_data.append(load_data_prov)              

            workbook = xlsxwriter.Workbook(file_path11)
            worksheet = workbook.add_worksheet()
            
            for row_num, row_data in enumerate(load_data):
                for col_num, cell_data in enumerate(row_data):
                    worksheet.write(row_num, col_num, cell_data)
            workbook.close()
            print("</> file accessed \n ---> data obtained ..... written correctly")
        else:
            pass


    elif(g_name == 'dMdH_show'):    
        # Data for the third plot
        #T_lin = np.delete(T_lin, -1)
        H_lin = np.delete(H_lin, -1)
        T_lin, H_lin = np.meshgrid(T_lin, H_lin)
        np_dMdH_con_correc = np.asarray(dMdH_con_correc_02)

        # Create the third plot
        fig = plt.figure()
        ax3 = fig.add_subplot(111, projection='3d')

        # Plot the surface for the second dataset
        surface3 = ax3.plot_surface(T_lin, H_lin, np_dMdH_con_correc, cmap='RdBu')
        ax3.set_xlabel(f'T {T_unit}')
        ax3.set_ylabel(f'H {H_unit}')
        ax3.set_zlabel(f'dM/dH ({M_unit})/{H_unit}')
        ax3.set_title('dM/dH distribution')
        cb = fig.colorbar(surface3, ax=ax3, label='dM/dH Value')
        ax = cb.ax
        text = ax.yaxis.label

        print ("</> request for dMdH_show ----> accepted & generated using the corrected form ([np_dMdH_con_correc]) with cmap = 'RdBu'")
        # Show the third plot
        plt.show()

        if save_data == 'allow':

            workbook = openpyxl.Workbook()

            # Create a new worksheet
            worksheet = workbook.active

            file_path10 = os.path.join(folder_path, 'mce_output', 'obtained_dMdH_show.xlsx')
            workbook.save(file_path10)
            print ("</> workbook.save ----> [file_path10]")
            workbook.close()

            print ("</> workbook ----> closed")

            T_for_sheet = T_lin[0].tolist()
            H_for_sheet = []
            for i in range(0, len(H_lin)):
                H_for_sheet.append(H_lin[i][0])
            load_data = []
            T_for_sheet.insert(0, 'Temperature(T)')
            T_for_sheet.insert(1, '')
            ss_lst = ['↓'] * len(T_lin[0])
            ss_lst.insert(0, 'Field(H)')
            ss_lst.insert(1, '')

            load_data.append(T_for_sheet)
            load_data.append(ss_lst)

            
            for i in range(0, len(np_dMdH_con_correc)):
                load_data_prov = []
                load_data_prov = np_dMdH_con_correc[i].tolist()
                load_data_prov.insert(0, H_for_sheet[i])
                load_data_prov.insert(1, '→')  

                load_data.append(load_data_prov)              

            workbook = xlsxwriter.Workbook(file_path10)
            worksheet = workbook.add_worksheet()
            
            for row_num, row_data in enumerate(load_data):
                for col_num, cell_data in enumerate(row_data):
                    worksheet.write(row_num, col_num, cell_data)
            workbook.close()
            print("</> file accessed \n ---> data obtained ..... written correctly")
        else:
            pass

    elif(g_name == 'dMdT_show'):    
        # Data for the fourth plot
        T_lin = np.delete(T_lin, -1)
        #H_lin = np.delete(H_lin, -1)
        T_lin, H_lin = np.meshgrid(T_lin, H_lin)
        np_dMdT_con_correc = np.asarray(dMdT_con)

        # Create the fourth plot
        fig = plt.figure()
        ax5 = fig.add_subplot(111, projection='3d')

        # Plot the surface for the second dataset
        surface5 = ax5.plot_surface(T_lin, H_lin, np_dMdT_con_correc, cmap='BrBG')
        ax5.set_xlabel(f'T {T_unit}')
        ax5.set_ylabel(f'H {H_unit}')
        ax5.set_zlabel(f'dM/dT ({M_unit})/{T_unit}')
        ax5.set_title('dM/dT distribution')
        cb = fig.colorbar(surface5, ax=ax5, label='dM/dT Value')
        ax = cb.ax
        text = ax.yaxis.label

        print ("</> request for dMdT_show ----> accepted & generated using the corrected form ([np_dMdT_con_correc]) with cmap = 'BrBG'")


        #print ("T_lin", T_lin,"\nH_lin", H_lin,"\ndM/dT", np_dMdT_con_correc)
        # Show the fourth plot
        plt.show()

        if save_data == 'allow':
            workbook = openpyxl.Workbook()

            # Create a new worksheet
            worksheet = workbook.active

            file_path09 = os.path.join(folder_path, 'mce_output', 'obtained_dMdT_show.xlsx')
            workbook.save(file_path09)
            print ("</> workbook.save ----> [file_path09]")
            workbook.close()

            print ("</> workbook ----> closed")

            T_for_sheet = T_lin[0].tolist()
            H_for_sheet = []
            for i in range(0, len(H_lin)):
                H_for_sheet.append(H_lin[i][0])
            load_data = []
            T_for_sheet.insert(0, 'Temperature(T)')
            T_for_sheet.insert(1, '')
            ss_lst = ['↓'] * len(T_lin[0])
            ss_lst.insert(0, 'Field(H)')
            ss_lst.insert(1, '')

            load_data.append(T_for_sheet)
            load_data.append(ss_lst)

            
            for i in range(0, len(np_dMdT_con_correc)):
                load_data_prov = []
                load_data_prov = np_dMdT_con_correc[i].tolist()
                load_data_prov.insert(0, H_for_sheet[i])
                load_data_prov.insert(1, '→')  

                load_data.append(load_data_prov)              

            workbook = xlsxwriter.Workbook(file_path09)
            worksheet = workbook.add_worksheet()
            
            for row_num, row_data in enumerate(load_data):
                for col_num, cell_data in enumerate(row_data):
                    worksheet.write(row_num, col_num, cell_data)
            workbook.close()
            print("</> file accessed \n ---> data obtained ..... written correctly")
        else:
            pass

    elif(g_name == 'd2MdTH_show'):    
        # Data for the fourth plot
        T_lin = np.delete(T_lin, -1)
        H_lin = np.delete(H_lin, -1)
        T_lin, H_lin = np.meshgrid(T_lin, H_lin)
        np_dMdTH_con_correc = np.asarray(dMdTH_con_correc_02)

        # Create the fourth plot
        fig = plt.figure()
        ax6 = fig.add_subplot(111, projection='3d')

        # Plot the surface for the second dataset
        surface6 = ax6.plot_surface(T_lin, H_lin, np_dMdTH_con_correc, cmap='PuOr')
        ax6.set_xlabel(f'T {T_unit}')
        ax6.set_ylabel(f'H {H_unit}')
        ax6.set_zlabel(f"$\\frac{{d^2M}}{{dT \\cdot dH}} ({M_unit}/{T_unit}.{H_unit})$")
        ax6.set_title('d2M/(dT.dH) distribution')
        cb = fig.colorbar(surface6, ax=ax6, label='d2M/(dT.dH) Value')
        ax = cb.ax
        text = ax.yaxis.label

        print ("</> request for dMdTH_show ----> accepted & generated using the corrected form ([np_dMdTH_con_correc]) with cmap = 'PuOr'")


        #print ("T_lin", T_lin,"\nH_lin", H_lin,"\ndM/dT", np_dMdT_con_correc)
        # Show the fourth plot
        plt.show()

        if save_data == 'allow':

            workbook = openpyxl.Workbook()

            # Create a new worksheet
            worksheet = workbook.active

            file_path12= os.path.join(folder_path, 'mce_output', 'obtained_d2MdTH_show.xlsx')
            workbook.save(file_path12)
            print ("</> workbook.save ----> [file_path12]")


            # Close the workbook
            workbook.close()

            print ("</> workbook ----> closed")

            T_for_sheet = T_lin[0].tolist()
            H_for_sheet = []
            for i in range(0, len(H_lin)):
                H_for_sheet.append(H_lin[i][0])
            load_data = []
            T_for_sheet.insert(0, 'Temperature(T)')
            T_for_sheet.insert(1, '')
            ss_lst = ['↓'] * len(T_lin[0])
            ss_lst.insert(0, 'Field(H)')
            ss_lst.insert(1, '')

            load_data.append(T_for_sheet)
            load_data.append(ss_lst)

            
            for i in range(0, len(np_dMdTH_con_correc)):
                load_data_prov = []
                load_data_prov = np_dMdTH_con_correc[i].tolist()
                load_data_prov.insert(0, H_for_sheet[i])
                load_data_prov.insert(1, '→')  

                load_data.append(load_data_prov)              

            workbook = xlsxwriter.Workbook(file_path12)
            worksheet = workbook.add_worksheet()
            
            for row_num, row_data in enumerate(load_data):
                for col_num, cell_data in enumerate(row_data):
                    worksheet.write(row_num, col_num, cell_data)
            workbook.close()
            print("</> file accessed \n ---> data obtained ..... written correctly")
        else:
            pass



    else:
        print ("xxx ---> g_name not found")
    
    return ''


import xlsxwriter
import openpyxl
import os

def data_writing(g_name, file_dir, n, T, Label_one, six_entropy_change_con, M_sqr, one_H_by_M_con, M_pow_MFT, H_by_M_pow_MFT, M_pow_TMFT, H_by_M_pow_TMFT, M_pow_3DH, H_by_M_pow_3DH, M_pow_3DI, H_by_M_pow_3DI, susceptibility_final, T_FWHM_con_final, RCP_final, H_for_RCP, N_expo_con, T_arr, N_H_label):
    """
    Store data to two separate Excel spreadsheets.

    Args:
        n (int): Number of data points.
        T (list): List of temperature values.
        one_H_by_M_con (list): List of lists containing H/M values for each temperature.
        M_sqr (list): List of lists containing M^2 values for each temperature.
        six_entropy_change_con (list): List of lists containing entropy change values for each temperature.

    Note:
        The function will insert appropriate headers and blank spaces in the data before saving to Excel.
    """

    # Specify the folder where you want to save the Excel file
    folder_path = os.path.dirname(file_dir)



    # Write data to the worksheet
    #worksheet['A1'] = 'Hello'
    #worksheet['B1'] = 'World'

    new_folder_name = 'mce_output'
    new_folder_path = os.path.join(folder_path, new_folder_name)

    # Check if the folder already exists, and create it if not
    if not os.path.exists(new_folder_path):
        os.makedirs(new_folder_path)
        print(f"</> folder '{new_folder_name}' created at '{new_folder_path}'")
    else:
        print(f"</> folder '{new_folder_name}' already exists at '{new_folder_path}'")







    if (g_name== 'dSmT_plot' or g_name== 'all_plots'): 

        # Create a new Excel workbook
        workbook = openpyxl.Workbook()

        # Create a new worksheet
        worksheet = workbook.active
        # Create the full file path including the folder and filename
        file_path01 = os.path.join(folder_path, 'mce_output', 'obtained_delSm_data.xlsx')
        workbook.save(file_path01)
        print ("</> workbook.save ----> [file_path01]")  
        workbook.close()

        print ("</> workbook ----> closed")

        # Save entropy change data to the first Excel spreadsheet (path_two).
        workbook = xlsxwriter.Workbook(file_path01)
        worksheet = workbook.add_worksheet()
        row = 0
        six_entropy_change_con[0].insert(0, 'Field(H)')
        six_entropy_change_con[0].insert(1, 'Temperature(T)')
        for i in range (1, len(six_entropy_change_con)):
            six_entropy_change_con[i].insert(0, Label_one[i-1])
            six_entropy_change_con[i].insert(1, '↓') 

        for col, data in enumerate(six_entropy_change_con):
            worksheet.write_column(row, col, data)
        workbook.close()
        print("</> file accessed \n ---> data obtained ..... written correctly")
    else:
        pass





    # Save M^2 vs. H/M data to the second Excel spreadsheet (path_three).
    if (g_name== 'MFT_plot' or g_name== 'all_plots'):

        # Create a new Excel workbook
        workbook = openpyxl.Workbook()

        # Create a new worksheet
        worksheet = workbook.active
        file_path02 = os.path.join(folder_path, 'mce_output', 'obtained_modified_arrott_plot01_data.xlsx')
        workbook.save(file_path02)
        print ("</> workbook.save ----> [file_path02]")
        workbook.close()

        print ("</> workbook ----> closed")


        # Prepare M^2 vs. H/M data before saving to the second Excel spreadsheet (path_three).
        for i in range(0, 2 * n, 1):
            lo = 2 * i + 1
            T.insert(lo, '   ')


        MFT = H_by_M_pow_MFT.tolist()
        M_pow_MFT_tolist = M_pow_MFT.tolist()

        for i in range(0, n, 1):
            x_index = 2 * i + 1
            MFT.insert(x_index, M_pow_MFT_tolist[i])

        for i in range(0, 2 * n, 1):
            MFT[i].insert(0, T[i])
            MFT[i].insert(1, '↓')
            if i % 2 == 0:
                MFT[i].insert(2, 'H/M')
            else:
                MFT[i].insert(2, 'M^2')


        workbook = xlsxwriter.Workbook(file_path02)
        worksheet = workbook.add_worksheet()
        row = 0
        for col, data in enumerate(MFT):
            worksheet.write_column(row, col, data)
        workbook.close()
        print("</> file accessed \n ---> data obtained ..... written correctly")
    else:
        pass

    if (g_name== 'arrott_plots' or g_name== 'all_plots'):
        # Create a new Excel workbook
        workbook = openpyxl.Workbook()

        # Create a new worksheet
        worksheet = workbook.active

        file_path03 = os.path.join(folder_path, 'mce_output', 'obtained_modified_arrott_plot02_data.xlsx')
        file_path04 = os.path.join(folder_path, 'mce_output', 'obtained_modified_arrott_plot03_data.xlsx')
        file_path05 = os.path.join(folder_path, 'mce_output', 'obtained_modified_arrott_plot04_data.xlsx')
        workbook.save(file_path03)
        print ("</> workbook.save ----> [file_path03]")
        workbook.save(file_path04)
        print ("</> workbook.save ----> [file_path04]")
        workbook.save(file_path05)
        print ("</> workbook.save ----> [file_path05]")

        # Close the workbook
        workbook.close()

        print ("</> workbook ----> closed")
        
        TMFT = H_by_M_pow_TMFT.tolist()
        M_pow_TMFT_tolist = M_pow_TMFT.tolist()

        for i in range(0, n, 1):
            x_index = 2 * i + 1
            TMFT.insert(x_index, M_pow_TMFT_tolist[i])

        for i in range(0, 2 * n, 1):
            TMFT[i].insert(0, T[i])
            TMFT[i].insert(1, '↓')
            if i % 2 == 0:
                TMFT[i].insert(2, 'H/M')
            else:
                TMFT[i].insert(2, 'M^(1/0.25)')

 
        workbook = xlsxwriter.Workbook(file_path03)
        worksheet = workbook.add_worksheet()
        row = 0
        for col, data in enumerate(TMFT):
            worksheet.write_column(row, col, data)
        workbook.close()    

        D3DH = H_by_M_pow_3DH.tolist()
        M_pow_3DH_tolist = M_pow_3DH.tolist()

        for i in range(0, n, 1):
            x_index = 2 * i + 1
            D3DH.insert(x_index, M_pow_3DH_tolist[i])

        for i in range(0, 2 * n, 1):
            D3DH[i].insert(0, T[i])
            D3DH[i].insert(1, '↓')
            if i % 2 == 0:
                D3DH[i].insert(2, '(H/M)^(1/1.336)')
            else:
                D3DH[i].insert(2, 'M^(1/0.365)')


        workbook = xlsxwriter.Workbook(file_path04)
        worksheet = workbook.add_worksheet()
        row = 0
        for col, data in enumerate(D3DH):
            worksheet.write_column(row, col, data)
        workbook.close()

        D3DI = H_by_M_pow_3DI.tolist()
        M_pow_3DI_tolist = M_pow_3DI.tolist()

        for i in range(0, n, 1):
            x_index = 2 * i + 1
            D3DI.insert(x_index, M_pow_3DI_tolist[i])

        for i in range(0, 2 * n, 1):
            D3DI[i].insert(0, T[i])
            D3DI[i].insert(1, '↓')
            if i % 2 == 0:
                D3DI[i].insert(2, '(H/M)^(1/1.24)')
            else:
                D3DI[i].insert(2, 'M^(1/0.325)')


        workbook = xlsxwriter.Workbook(file_path05)
        worksheet = workbook.add_worksheet()
        row = 0
        for col, data in enumerate(D3DI):
            worksheet.write_column(row, col, data)
        workbook.close()
        print("</> file accessed \n ---> data obtained ..... written correctly")
    else:
        pass
    if (g_name== 'sus_plot' or g_name== 'all_plots'):

        # Create a new Excel workbook
        workbook = openpyxl.Workbook()

        # Create a new worksheet
        worksheet = workbook.active

        file_path06 = os.path.join(folder_path, 'mce_output', 'obtained_susceptibility_data.xlsx')
        workbook.save(file_path06)
        print ("</> workbook.save ----> [file_path06]")
        workbook.close()

        print ("</> workbook ----> closed")        
        workbook = xlsxwriter.Workbook(file_path06)
        worksheet = workbook.add_worksheet()
        
        for row_num, row_data in enumerate(susceptibility_final):
            for col_num, cell_data in enumerate(row_data):
                worksheet.write(row_num, col_num, cell_data)
        workbook.close()
        print("</> file accessed \n ---> data obtained ..... written correctly")
    else:
        pass


    if (g_name== 'RCP_plot' or g_name== 'all_plots'):

        # Create a new Excel workbook
        workbook = openpyxl.Workbook()

        # Create a new worksheet
        worksheet = workbook.active

        file_path07 = os.path.join(folder_path, 'mce_output', 'obtained_RCP.xlsx')

        # Save the workbook to the specified folder and file


        workbook.save(file_path07)
        print ("</> workbook.save ----> [file_path07]")

        # Close the workbook
        workbook.close()

        print ("</> workbook ----> closed")


        RCP_sheet = [['Magnetic Field','', 'T_FWHM', 'RCP'], ['↓', '', '↓', '↓']]

        
        for i in range(0, len(H_for_RCP)):
            RCP_sheet_prov = []
            RCP_sheet_prov.append(H_for_RCP[i])
            RCP_sheet_prov.append('→')  
            RCP_sheet_prov.append(T_FWHM_con_final[i])
            RCP_sheet_prov.append(RCP_final[i])  
            RCP_sheet.append(RCP_sheet_prov)
        
        workbook = xlsxwriter.Workbook(file_path07)
        worksheet = workbook.add_worksheet()
        
        for row_num, row_data in enumerate(RCP_sheet):
            for col_num, cell_data in enumerate(row_data):
                worksheet.write(row_num, col_num, cell_data)
        workbook.close()
        print("</> file accessed \n ---> data obtained ..... written correctly")
    else:
        pass

    if (g_name== 'N_plot' or g_name== 'all_plots'):
        # Create a new Excel workbook
        workbook = openpyxl.Workbook()

        # Create a new worksheet
        worksheet = workbook.active

        file_path08 = os.path.join(folder_path, 'mce_output', 'obtained_N_Exponent.xlsx')

        # Save the workbook to the specified folder and file


        workbook.save(file_path08)
        print ("</> workbook.save ----> [file_path08]")

        # Close the workbook
        workbook.close()

        print ("</> workbook ----> closed")  

        arrow_n = []
        for i in range (0, len(T_arr)):
                arrow_n.append('↓')

        T_arr.insert(0, 'Temperature(T)')
        T_arr.insert(1, '')
        arrow_n.insert(0, 'Field(H)')
        arrow_n.insert(1, '')

        N_expo_save = N_expo_con

        N_expo_save_final = []
        N_expo_save_final.append(T_arr)
        N_expo_save_final.append(arrow_n)

        for i in range (0, len(N_expo_save)):
                
                N_expo_save[i].insert(0, N_H_label[i])
                N_expo_save[i].insert(1, '→')
                N_expo_save_final.append(N_expo_save[i])
        
        workbook = xlsxwriter.Workbook(file_path08)
        worksheet = workbook.add_worksheet()
        
        for row_num, row_data in enumerate(N_expo_save_final):
            for col_num, cell_data in enumerate(row_data):
                worksheet.write(row_num, col_num, cell_data)
        workbook.close()
        print("</> file accessed \n ---> data obtained ..... written correctly")  
    else:
        pass

    return



import numpy as np
import matplotlib.pyplot as plt
import itertools

def data_plotting(g_name, one_n, n, T, H, M_unit, H_unit, T_unit, colour, marker, Label_one, plot_legend, loc, one_M_plot_final, two_M_plot_final, H_plot_final, temperatures, five_entropy_change_con, M_sqr, one_H_by_M_con, N_expo_con, T_arr, N_H_label, T_sus, sus_inv, sus, np_T_sus, regression_line, np_T_sus_bu, regression_bu, x, linear_threshold, T_FWHM_con, T_FWHM_con_final, RCP_con, RCP_final, H_for_RCP, samp_name):
    """
    Visualize the data using Matplotlib.

    Args:
        one_n (int): Number of data points along one direction.
        n (int): Total number of data points.
        T (list): List of temperature values.
        H (list): List of external field values.
        colour (list): List of color strings for plotting.
        marker (itertools.cycle): Iterator over marker symbols for plotting.
        Label_one (list): List of labels for the legend.
        plot_legend (str): Flag to indicate whether to show legends in the plots ('YES' or 'NO').
        one_M_plot_final (list): List of lists containing magnetization data grouped by one direction.
        two_M_plot_final (list): List of lists containing magnetization data grouped by two directions.
        H_plot_final (list): List of external field values without the last one.
        temperatures (list): List of temperature values for plotting.
        five_entropy_change_con (list): List of lists containing entropy change data.
        M_sqr (list): List of squared magnetization values (M^2).
        one_H_by_M_con (list): List of lists containing H/M (applied field / magnetization) data.

    Returns:
        None: The function displays plots based on the data.
    """

    if (g_name== 'MH_plot' or g_name== 'all_plots'):

        if plot_legend == 'Yes':        
            # Plotting magnetization vs. applied field for each temperature.
            for k in range(n):
                plt.plot(H_plot_final, one_M_plot_final[k], linestyle='solid', label=T[n - (k + 1)], marker=next(marker), markersize =5, linewidth= 0.5)
            plt.legend(loc=str(loc), frameon=False, ncol=3)
            plt.xlabel(f"Magnetic Field (H) {H_unit}")
            plt.ylabel(f"Magnetization (M) {M_unit}")
            plt.title("Field depedence of Magnetization")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()

        else:
            for k in range (0, n, 1):
              plt.plot(H_plot_final, one_M_plot_final[k], linestyle='solid', label = T[n-(k+1)], marker = next(marker), markersize =5, linewidth= 0.5)
             
            plt.xlabel(f"Magnetic Field (H) {H_unit}")
            plt.ylabel(f"Magnetization (M) {M_unit}")
            plt.title("Field depedence of Magnetization", fontname = "monospace")

            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
        print ("</> request for MH_plot ----> accepted & generated using [H_plot_final] & [one_M_plot_final]")

    if (g_name== 'MT_plot' or g_name== 'all_plots'):
        
        if plot_legend == 'Yes':
            # Plotting magnetization vs. temperature for selected external field values.
            for k in range(0, one_n - 1, int(one_n / 10)):
                plt.plot(temperatures, two_M_plot_final[k], linestyle='solid', label=(round((H[k] * (10 ** (-4))), 1)), marker=next(marker), markersize =5, linewidth= 0.5)
            plt.legend(loc=str(loc), frameon=False, ncol=2)
            plt.xlabel(f"Temperature ({T_unit})")
            plt.ylabel(f"Magnetization (M) {M_unit}")
            plt.title("Temperature depedence of Magnetization")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
        else:
            for k in range (0, (one_n - 1), (int(one_n/10))):
              plt.plot(temperatures, two_M_plot_final[k], linestyle='solid', label = (round((H[k]*(10**(-4))),1)), marker = next(marker), markersize =5, linewidth= 0.5)
              
            plt.xlabel(f"Temperature ({T_unit})")
            plt.ylabel(f"Magnetization (M) {M_unit}")
            plt.title("Temperature depedence of Magnetization", fontname = "monospace")

            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()

        print ("</> request for MT_plot ----> accepted & generated using [temperatures] & [two_M_plot_final]")

    if (g_name== 'dSmT_plot' or g_name== 'all_plots'):

        if plot_legend == 'Yes':        
            # Plotting entropy change vs. temperature for each temperature.
            for q in range(0, len(Label_one)):
                plt.plot(temperatures, five_entropy_change_con[q], linestyle='solid', label=Label_one[q], color= next(colour), marker=next(marker), markersize =5, linewidth= 0.5)
                plt.legend(loc=str(loc), frameon=False, ncol=2)
            plt.xlabel(f"Temperature ({T_unit})")
            plt.ylabel(f"∆Sm ({M_unit}).{H_unit}/{T_unit}")
            plt.title("Temperature depedence of Entropy change")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
        else:
            for q in range(0, len(Label_one), 1):
              plt.plot((temperatures), (five_entropy_change_con[q]), linestyle='solid', color= next(colour), marker = next(marker), markersize =5, linewidth= 0.5)
                              
            plt.xlabel(f"Temperature ({T_unit})", fontname = "monospace")
            plt.ylabel(f"∆Sm ({M_unit}).{H_unit}/{T_unit}", fontname = "monospace")
            plt.title("Temperature depedence of Entropy change", fontname = "monospace")

            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show() 

        print ("</> request for dSmT_plot ----> accepted & generated using [temperatures] & [five_entropy_change_con]")           

    if (g_name== 'N_plot' or g_name== 'all_plots'):
        if plot_legend == 'Yes':        

            for i in range (0, len(N_expo_con)):            
                    plt.plot(T_arr, N_expo_con[i], linestyle='solid', label=N_H_label[i], color= next(colour), marker=next(marker), markersize =5, linewidth= 0.5)
                    plt.legend(loc=str(loc), frameon=False, ncol=2)
            plt.xlabel(f"Temperature ({T_unit})")
            plt.ylabel(f"N exponent ", fontsize= 8)
            plt.title("Temperature depedence of N exponent")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()

        else:
            for i in range (0, count):            
                    plt.plot(T_arr, N_expo_con[i], linestyle='solid', label=N_H_label[i], color= next(colour), marker=next(marker), markersize =5, linewidth= 0.5)
                    
            plt.xlabel(f"Temperature ({T_unit})")
            plt.ylabel(f"N exponent")
            plt.title("Temperature depedence of N exponent")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
        print ("</> request for N_plot ----> accepted & generated using [T_arr] & [N_expo_con]")

    if (g_name== 'MFT_plot' or g_name== 'all_plots'):

        if plot_legend == 'Yes':
            # Plotting M^2 vs. H/M for each temperature.
            for i in range(n):
                plt.plot(one_H_by_M_con[i], M_sqr[i], linestyle='solid', label=T[n - (i + 1)], marker=next(marker), markersize =5, linewidth= 0.5)
            plt.legend(loc=str(loc), frameon=False, ncol=2)
            plt.xlabel(f"H/M (Applied Field / Magnetization) {H_unit}/({M_unit})")
            plt.ylabel(f"M^2 (Magnetization Square) {M_unit}^2")
            plt.title("Mean Field Model")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
        else:
            for i in range (0, n, 1):
             plt.plot(one_H_by_M_con[i], M_sqr[i], linestyle='solid', label = T[n-(i+1)], marker = next(marker), markersize =5, linewidth= 0.5)

            plt.xlabel(f"H/M (Applied Field / Magnetization) {H_unit}/({M_unit})", fontname = "monospace")
            plt.ylabel(f"M^2 (Magnetization Square) {M_unit}^2", fontname = "monospace")
            plt.title("Mean Field Model", fontname = "monospace")

            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
        print ("</> request for MFT_plot ----> accepted & generated using [one_H_by_M_con] & [M_sqr]")

    if (g_name== 'sus_plot' or g_name== 'all_plots'):

        label01 = f"1/Chi (inverse susceptibility)"
        label02 = f"Chi (susceptibility)"
        label03 = f"Regression (x= {str(round(x, 2))}, y=0)"
        label04 = f"1/Chi associated with Regression"
        


                    
        fig, ax1 = plt.subplots()
        ax1.set_xlabel(f"Temperature ({T_unit})")
        ax1.set_ylabel(f"1/Chi {H_unit}/({M_unit})")
        ax1.plot(T_sus, sus_inv, linestyle='solid', marker='o',label=label01, markerfacecolor='none', color='black',markersize=5, linewidth=0.5)
        if linear_threshold == 0.0 or linear_threshold == 'None':
            pass
        else:
            ax1.plot(np_T_sus, regression_line, linestyle= 'dashed', color='black', label=label03, linewidth=0.7)
            ax1.plot(np_T_sus_bu, regression_bu, linestyle='none', marker='o', markerfacecolor='green', label=label04, color='black',markersize=5, linewidth=0.5)
        ax1.tick_params(axis='y')

        ax2 = ax1.twinx()
        ax2.set_ylabel(f"Chi ({M_unit})/{H_unit}")
        ax2.plot(T_sus, sus, linestyle='solid', marker='o', label=label02, color='r', markersize=5, linewidth=0.5)
        ax2.tick_params(axis='y')

        # Adjust legend positioning


        # Use the bbox_to_anchor parameter to position the legends
        fig.legend(loc= "upper center", frameon=False, ncol=2)
        plt.title(f"Temperature depedence of Susceptibility and Inverse Susceptibility")
        plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
        plt.show()

        print ("</> request for sus_plot ----> accepted & generated ")

    """
    Plot Relative Cooling Power (RCP) and Full Width at Half Maximum (T_FWHM) against magnetic field (H).

    Args:
        T_FWHM_con (list): List of calculated FWHM values.
        Label_one (list): List of temperature values.
        RCP_con (list): List of calculated RCP values for all temperatures.
        RCP_final (list): List of RCP values with sufficient data.
        H_for_RCP (list): List of magnetic field values corresponding to RCP.
        samp_name (str): Name of the sample.

    Returns:
        None: The function only plots the RCP/T_FWHM vs H graphs.
    """
    if (g_name== 'RCP_plot' or g_name== 'all_plots'):
        samp_name_plus_RCP = "RCP (" + samp_name + ") :: max val : " + str(np.max(RCP_con)) + "(" + str(M_unit) + ")." + str(H_unit)
        samp_name_plus_T_FWHM = "T_FWHM (" + samp_name + ") :: max width : " + str(np.max(T_FWHM_con)) + str(T_unit)

        if len(RCP_final) >= 2:
            fig, ax1 = plt.subplots()
            ax1.set_xlabel(f"Magnetic Field ({H_unit})")
            ax1.set_ylabel(f"RCP ({M_unit}).{H_unit}")
            ax1.plot(H_for_RCP, RCP_final, linestyle='solid', marker='o', label=samp_name_plus_RCP, color='r', markersize=5, linewidth=0.5)
            ax1.legend(loc='upper left', frameon=False, ncol=2)
            ax1.tick_params(axis='y')

            ax2 = ax1.twinx()
            ax2.set_ylabel(f"T_FWHM ({T_unit})")
            ax2.plot(H_for_RCP, T_FWHM_con_final, linestyle='-', marker='+', label=samp_name_plus_T_FWHM, color='black', markersize=7, linewidth=0.0)
            ax2.legend(loc='lower right', frameon=False, ncol=2)
            ax2.tick_params(axis='y')

            plt.title("Field depedence of RCP/T_FWHM")
            plt.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
            plt.show()
            print ("</> request for RCP_plot ----> accepted & generated ")
        else:
            print ("</> request for RCP_plot ----> denied for insufficient data ")

    return



import numpy as np

def T_FWHM_RCP(n, Label_one, six_entropy_change_con):
    """
    Calculate Full Width at Half Maximum (FWHM) and Relative Cooling Power (RCP) from entropy data.

    Args:
        n (int): Number of data points.
        Label_one (list): List of temperature values.
        six_entropy_change_con (list): List of lists containing entropy change data.

    Returns:
        tuple: A tuple containing four lists: 
               1. T_FWHM_con: List of calculated FWHM values.
               2. RCP_con: List of calculated RCP values.
               3. RCP_final: List of RCP values with sufficient data.
               4. H_for_RCP: List of H values corresponding to RCP.
    """
    T_FWHM_con = []
    RCP_con = []
    RCP_error = []
    RCP_final = []
    T_FWHM_con_final = []
    H_for_RCP = []

    # Iterate through each temperature data.
    for j in range(1, len(Label_one) + 1):
        del_S_peak = np.min(six_entropy_change_con[j])

        # Find the index of the maximum entropy change value.
        for k in range(n - 1):
            if six_entropy_change_con[j][k] == del_S_peak:
                kth_index = k
                max_entropy_at_T = six_entropy_change_con[0][kth_index]

        half_max = del_S_peak / 2
        half_max_entropy_at_T_con = [six_entropy_change_con[0][0]]

        # Find the temperatures where the entropy change is approximately half of the maximum value.
        for i in range(n - 2):
            i_th = six_entropy_change_con[j][i]
            i_th_plus_one = six_entropy_change_con[j][i + 1]
            if (i_th_plus_one >= half_max >= i_th) or (i_th_plus_one <= half_max <= i_th):
                T_i_th = six_entropy_change_con[0][i]
                T_i_th_plus_one = six_entropy_change_con[0][i + 1]
                del_S_dif = abs(i_th_plus_one - i_th)
                T_dif = T_i_th_plus_one - T_i_th
                dif_bet_ith_half = abs(half_max - i_th)
                half_max_entropy_at_T = (T_dif / del_S_dif) * dif_bet_ith_half + T_i_th
                half_max_entropy_at_T_con.append(abs(half_max_entropy_at_T))
        half_max_entropy_at_T_con.append(six_entropy_change_con[0][n - 2])

        # Find the temperature range for FWHM
        for l in range(len(half_max_entropy_at_T_con) - 1):
            l_th = half_max_entropy_at_T_con[l]
            l_th_plus_one = half_max_entropy_at_T_con[l + 1]
            if l_th <= max_entropy_at_T and l_th_plus_one >= max_entropy_at_T:
                T_left = l_th
                T_right = l_th_plus_one

        T_FWHM = T_right - T_left
        RCP = abs(T_FWHM) * abs(del_S_peak)
        T_FWHM_con.append(round(T_FWHM, 4))
        RCP_con.append(round(RCP, 4))

        # Detect insufficient data for RCP calculation.
        if (T_left == six_entropy_change_con[0][0] != 0.0) or (T_right == six_entropy_change_con[0][n - 2]):
            RCP_error.append(Label_one[j - 1])
        else:
            T_FWHM_con_final.append(T_FWHM)
            RCP_final.append(round(RCP, 4))
            H_for_RCP.append(Label_one[j - 1])

    lst_RCP_error = len(RCP_error) - 1

    if len(RCP_error) == 1:
        print("</> insufficient data ----> error detected while calculating RCP at H : {} T".format(RCP_error[0]))
    elif len(RCP_error) >= 1:
        print("</> insufficient data ----> error detected while calculating RCP from H : {} T to H : {} T".format(RCP_error[0], RCP_error[lst_RCP_error]))
    else:
        print('')

    return T_FWHM_con, T_FWHM_con_final, RCP_con, RCP_final, H_for_RCP


from .mce_2D_Operations.data_2d_operation import mce
from .mce_3D_Distribution.data_3d_plotting import mce_3d
from .MH_Interpolation.interpol import interpol

__all__ = [
    'interpol',
    'mce',
    'mce_3d'
]
# magnetocaloric package
# Developed by Supratim Das
# First release: November 5, 2021
# Email: supratim0707@gmail.com

# This file can include package-level metadata or comments.
# It's executed when the package is imported.

# Example metadata:

__version__ = '1.6.6'  # Package version
__author__ = 'Supratim Das' # Package author
__email__ = 'supratim0707@gmail.com' #Get in Touch
__description__ = 'Effective Approach To Calculate Magnetocaloric Effect Of Any Magnetic Material Using Python'  # Package description
__license__ = 'MIT'  # License information
__url__ = 'https://pypi.org/project/magnetocaloric/'  # URL to the package's repository
__keywords__ = ['magnetism', 'thermodynamics', 'magnetocaloric']  # Keywords describing the package
